<style>
  body {
    font-family: Times New Roman, "Helvetica Neue", Meiryo, sans-serif;
    margin: 0;
  }
  span.title {
    text-align: center;
    font-size: 2.5em;
    font-weight: bold;
  }
  h2.head {
    border-bottom: 2px solid #d5d5d5;
    font-size: 1.5em;
    margin: 0.75em 0;
  }
  h3.subhead {
    font-size: 1.1em;
    margin: 0.75em 0;
    position: relative;
    padding: 0 1em;
    display: inline-block;
    top:0;
  }
  h3.subhead:before, h3.subhead:after { 
    position: absolute;
    top: 0;
    content:'';
    width: 8px;
    height: 100%;
    display: inline-block;
  }
  h3.subhead:before {
    border-top: solid 1px #d5d5d5;
    border-left: solid 1px #d5d5d5;
    border-bottom: solid 1px #d5d5d5;
    left: 0;
  }
  h3.subhead:after {
    content: '';
    border-top: solid 1px #d5d5d5;
    border-right: solid 1px #d5d5d5;
    border-bottom: solid 1px #d5d5d5;
    right: 0;
  }
  table {
    width: 100%;
    border-collapse: collapse;
    table-layout: fixed;
  }
  thead {
    font-size: 1.15em;
    font-weight: bold;
  }
  td, th {
    width: 50%;
    vertical-align: top;
  }
  .en {
    line-height: 1.6em;
  }
  .ja {
    font-size: 0.85em;
    line-height: 2.25em;
  }
  p.img_center {
    text-align: center;
    border: 5px solid #d5d5d5;
    padding: 20px 10px;
    max-width: 100%;
    margin: 10px 10%;
  }
  img {
    max-width: 80%;
  }
</style>