# coding: utf-8

__copyright__       = "Copyright (C) 2020 Shuto Iwasaki"
__version__         = "3.5.2"

__license__         = "MIT"
__author__          = "Shuto Iwasaki"
__author_twitter__  = "https://twitter.com/cabernet_rock"
__author_email__    = "cabernet.rock@gmail.com"
__url__             = "https://github.com/iwasakishuto/Translation-Gummy"

from . import gateways
from . import journals
from . import models
from . import translators
from .models import TranslationGummy